@@ -524,6 +524,8 @@ struct _vte_capability_string _vte_xterm_capability_strings[] = {
 	{OSC "2;%s" BEL, "set-window-title", 0},
 	{OSC "3;%s" BEL, "set-xproperty", 0},
 	{OSC "4;%s" BEL, "change-color", 0},
+        {OSC "6;%s" BEL, "set-current-file-uri", 0},
+        {OSC "7;%s" BEL, "set-current-directory-uri", 0},
 	{OSC "10;%s" BEL, "change-foreground-colors", 0},
 	{OSC "11;%s" BEL, "change-background-colors", 0},
 	{OSC "12;%s" BEL, "change-cursor-colors", 0},
@@ -543,6 +545,8 @@ struct _vte_capability_string _vte_xterm_capability_strings[] = {
 	{OSC "2;%s" ST, "set-window-title", 0},
 	{OSC "3;%s" ST, "set-xproperty", 0},
 	{OSC "4;%s" ST, "change-color", 0},
+        {OSC "6;%s" ST, "set-current-file-uri", 0},
+        {OSC "7;%s" ST, "set-current-directory-uri", 0},
 	{OSC "10;%s" ST, "change-foreground-colors", 0},
 	{OSC "11;%s" ST, "change-background-colors", 0},
 	{OSC "12;%s" ST, "change-cursor-colors", 0},
