@@ -570,13 +570,13 @@ font_info_find_for_context (PangoContext *context)
 		_vte_debug_print (VTE_DEBUG_PANGOCAIRO,
 				  "vtepangocairo: %p found font_info in cache\n",
 				  info);
-		return font_info_reference (info);
+		info = font_info_reference (info);
+	} else {
+		info = font_info_allocate (context);
+		info->ref_count = 1;
+		font_info_register (info);
 	}
 
-	info = font_info_allocate (context);
-	info->ref_count = 1;
-	font_info_register (info);
-
 	g_object_unref (context);
 
 	return info;
@@ -744,13 +744,21 @@ font_info_get_unistr_info (struct font_info *info,
 	return uinfo;
 }
 
+guint _vte_draw_get_style(gboolean bold, gboolean italic) {
+	guint style = 0;
+	if (bold)
+		style |= VTE_DRAW_BOLD;
+	if (italic)
+		style |= VTE_DRAW_ITALIC;
+	return style;
+}
+
 struct _vte_draw {
 	GtkWidget *widget;
 
 	gint started;
 
-	struct font_info *font;
-	struct font_info *font_bold;
+	struct font_info *fonts[4];
 	cairo_pattern_t *bg_pattern;
 
 	cairo_t *cr;
@@ -770,9 +778,16 @@ _vte_draw_new (GtkWidget *widget)
 	return draw;
 }
 
+cairo_t *
+_vte_draw_get_context (struct _vte_draw *draw)
+{
+  return draw->cr;
+}
+
 void
 _vte_draw_free (struct _vte_draw *draw)
 {
+	gint style;
 	_vte_debug_print (VTE_DEBUG_DRAW, "draw_free\n");
 
 	if (draw->bg_pattern != NULL) {
@@ -780,9 +795,13 @@ _vte_draw_free (struct _vte_draw *draw)
 		draw->bg_pattern = NULL;
 	}
 
-	if (draw->font != NULL) {
-		font_info_destroy (draw->font);
-		draw->font = NULL;
+	/* Free all fonts (make sure to destroy every font only once)*/
+	for (style = 3; style >= 0; style--) {
+		if (draw->fonts[style] != NULL &&
+			(style == 0 || draw->fonts[style] != draw->fonts[style-1])) {
+			font_info_destroy (draw->fonts[style]);
+			draw->fonts[style] = NULL;
+		}
 	}
 
 	if (draw->widget != NULL) {
@@ -915,29 +934,68 @@ _vte_draw_set_text_font (struct _vte_draw *draw,
 			const PangoFontDescription *fontdesc,
 			VteTerminalAntiAlias antialias)
 {
-	PangoFontDescription *bolddesc = NULL;
+	PangoFontDescription *bolddesc   = NULL;
+	PangoFontDescription *italicdesc = NULL;
+	PangoFontDescription *bolditalicdesc = NULL;
+	gint style, normal, bold, ratio;
 
 	_vte_debug_print (VTE_DEBUG_DRAW, "draw_set_text_font (aa=%d)\n",
 			  antialias);
 
-	if (draw->font_bold != draw->font)
-		font_info_destroy (draw->font_bold);
-	font_info_destroy (draw->font);
-	draw->font = font_info_create_for_widget (draw->widget, fontdesc, antialias);
+	/* Free all fonts (make sure to destroy every font only once)*/
+	for (style = 3; style >= 0; style--) {
+		if (draw->fonts[style] != NULL &&
+			(style == 0 || draw->fonts[style] != draw->fonts[style-1])) {
+			font_info_destroy (draw->fonts[style]);
+			draw->fonts[style] = NULL;
+		}
+	}
 
 	/* calculate bold font desc */
 	bolddesc = pango_font_description_copy (fontdesc);
 	pango_font_description_set_weight (bolddesc, PANGO_WEIGHT_BOLD);
 
-	draw->font_bold = font_info_create_for_widget (draw->widget, bolddesc, antialias);
+	/* calculate italic font desc */
+	italicdesc = pango_font_description_copy (fontdesc);
+	pango_font_description_set_style (italicdesc, PANGO_STYLE_ITALIC);
+
+	/* calculate bold italic font desc */
+	bolditalicdesc = pango_font_description_copy (bolddesc);
+	pango_font_description_set_style (bolditalicdesc, PANGO_STYLE_ITALIC);
+
+	draw->fonts[VTE_DRAW_NORMAL]  = font_info_create_for_widget (draw->widget,
+										fontdesc, antialias);
+	draw->fonts[VTE_DRAW_BOLD]    = font_info_create_for_widget (draw->widget,
+										bolddesc, antialias);
+	draw->fonts[VTE_DRAW_ITALIC]  = font_info_create_for_widget (draw->widget,
+										italicdesc, antialias);
+	draw->fonts[VTE_DRAW_ITALIC | VTE_DRAW_BOLD] =
+									font_info_create_for_widget (draw->widget,
+										bolditalicdesc, antialias);
 	pango_font_description_free (bolddesc);
+	pango_font_description_free (italicdesc);
+	pango_font_description_free (bolditalicdesc);
 
 	/* Decide if we should keep this bold font face, per bug 54926:
 	 *  - reject bold font if it is not within 10% of normal font width
 	 */
-	if ( abs((draw->font_bold->width * 100 / draw->font->width) - 100) > 10 ) {
-		font_info_destroy (draw->font_bold);
-		draw->font_bold = draw->font;
+	normal = VTE_DRAW_NORMAL;
+	bold   = normal | VTE_DRAW_BOLD;
+	ratio = draw->fonts[bold]->width * 100 / draw->fonts[normal]->width;
+	if (abs(ratio - 100) > 10) {
+		_vte_debug_print (VTE_DEBUG_DRAW,
+			"Rejecting bold font (%i%%).\n", ratio);
+		font_info_destroy (draw->fonts[bold]);
+		draw->fonts[bold] = draw->fonts[normal];
+	}
+	normal = VTE_DRAW_ITALIC;
+	bold   = normal | VTE_DRAW_BOLD;
+	ratio = draw->fonts[bold]->width * 100 / draw->fonts[normal]->width;
+	if (abs(ratio - 100) > 10) {
+		_vte_debug_print (VTE_DEBUG_DRAW,
+			"Rejecting italic bold font (%i%%).\n", ratio);
+		font_info_destroy (draw->fonts[bold]);
+		draw->fonts[bold] = draw->fonts[normal];
 	}
 }
 
@@ -945,41 +1003,41 @@ void
 _vte_draw_get_text_metrics(struct _vte_draw *draw,
 			   gint *width, gint *height, gint *ascent)
 {
-	g_return_if_fail (draw->font != NULL);
+	g_return_if_fail (draw->fonts[VTE_DRAW_NORMAL] != NULL);
 
 	if (width)
-		*width  = draw->font->width;
+		*width  = draw->fonts[VTE_DRAW_NORMAL]->width;
 	if (height)
-		*height = draw->font->height;
+		*height = draw->fonts[VTE_DRAW_NORMAL]->height;
 	if (ascent)
-		*ascent = draw->font->ascent;
+		*ascent = draw->fonts[VTE_DRAW_NORMAL]->ascent;
 }
 
 
 int
 _vte_draw_get_char_width (struct _vte_draw *draw, vteunistr c, int columns,
-			  gboolean bold)
+						  guint style)
 {
 	struct unistr_info *uinfo;
 
-	g_return_val_if_fail (draw->font != NULL, 0);
+	g_return_val_if_fail (draw->fonts[VTE_DRAW_NORMAL] != NULL, 0);
 
-	uinfo = font_info_get_unistr_info (bold ? draw->font_bold : draw->font, c);
+	uinfo = font_info_get_unistr_info (draw->fonts[style], c);
 	return uinfo->width;
 }
 
-static gboolean
-_vte_draw_has_bold (struct _vte_draw *draw)
+gboolean
+_vte_draw_has_bold (struct _vte_draw *draw, guint style)
 {
-	return (draw->font != draw->font_bold);
+	return (draw->fonts[style ^ VTE_DRAW_BOLD] != draw->fonts[style]);
 }
 
-static void
-set_source_color_alpha (cairo_t        *cr,
-			const PangoColor *color,
-			guchar alpha)
+void
+_vte_draw_set_source_color_alpha (struct _vte_draw *draw,
+                                  const PangoColor *color,
+                                  guchar            alpha)
 {
-	cairo_set_source_rgba (cr,
+	cairo_set_source_rgba (draw->cr,
 			      color->red / 65535.,
 			      color->green / 65535.,
 			      color->blue / 65535.,
@@ -989,17 +1047,17 @@ set_source_color_alpha (cairo_t        *cr,
 static void
 _vte_draw_text_internal (struct _vte_draw *draw,
 			 struct _vte_draw_text_request *requests, gsize n_requests,
-			 const PangoColor *color, guchar alpha, gboolean bold)
+			 const PangoColor *color, guchar alpha, guint style)
 {
 	gsize i;
 	cairo_scaled_font_t *last_scaled_font = NULL;
 	int n_cr_glyphs = 0;
 	cairo_glyph_t cr_glyphs[MAX_RUN_LENGTH];
-	struct font_info *font = bold ? draw->font_bold : draw->font;
+	struct font_info *font = draw->fonts[style];
 
 	g_return_if_fail (font != NULL);
 
-	set_source_color_alpha (draw->cr, color, alpha);
+	_vte_draw_set_source_color_alpha (draw, color, alpha);
 	cairo_set_operator (draw->cr, CAIRO_OPERATOR_OVER);
 
 	for (i = 0; i < n_requests; i++) {
@@ -1055,7 +1113,7 @@ _vte_draw_text_internal (struct _vte_draw *draw,
 void
 _vte_draw_text (struct _vte_draw *draw,
 	       struct _vte_draw_text_request *requests, gsize n_requests,
-	       const PangoColor *color, guchar alpha, gboolean bold)
+	       const PangoColor *color, guchar alpha, guint style)
 {
 	g_return_if_fail (draw->started);
 
@@ -1067,16 +1125,17 @@ _vte_draw_text (struct _vte_draw *draw,
 			g_string_append_unichar (string, requests[n].c);
 		}
 		str = g_string_free (string, FALSE);
-		g_printerr ("draw_text (\"%s\", len=%"G_GSIZE_FORMAT", color=(%d,%d,%d,%d), %s)\n",
-				str, n_requests, color->red, color->green, color->blue,
-				alpha, bold ? "bold" : "normal");
+		g_printerr ("draw_text (\"%s\", len=%"G_GSIZE_FORMAT", color=(%d,%d,%d,%d), %s - %s)\n",
+				str, n_requests, color->red, color->green, color->blue, alpha,
+				(style & VTE_DRAW_BOLD)   ? "bold"   : "normal",
+				(style & VTE_DRAW_ITALIC) ? "italic" : "regular");
 		g_free (str);
 	}
 
-	_vte_draw_text_internal (draw, requests, n_requests, color, alpha, bold);
+	_vte_draw_text_internal (draw, requests, n_requests, color, alpha, style);
 
 	/* handle fonts that lack a bold face by double-striking */
-	if (bold && !_vte_draw_has_bold (draw)) {
+	if ((style & VTE_DRAW_BOLD) && !_vte_draw_has_bold (draw, style)) {
 		gsize i;
 
 		/* Take a step to the right. */
@@ -1084,7 +1143,7 @@ _vte_draw_text (struct _vte_draw *draw,
 			requests[i].x++;
 		}
 		_vte_draw_text_internal (draw, requests,
-					   n_requests, color, alpha, FALSE);
+					   n_requests, color, alpha, style);
 		/* Now take a step back. */
 		for (i = 0; i < n_requests; i++) {
 			requests[i].x--;
@@ -1093,35 +1152,38 @@ _vte_draw_text (struct _vte_draw *draw,
 }
 
 gboolean
-_vte_draw_has_char (struct _vte_draw *draw, vteunistr c, gboolean bold)
+_vte_draw_has_char (struct _vte_draw *draw, vteunistr c, guint style)
 {
 	struct unistr_info *uinfo;
 
-	_vte_debug_print (VTE_DEBUG_DRAW, "draw_has_char ('0x%04X', %s)\n", c,
-			  bold ? "bold" : "normal");
+	_vte_debug_print (VTE_DEBUG_DRAW, "draw_has_char ('0x%04X', %s - %s)\n", c,
+				(style & VTE_DRAW_BOLD)   ? "bold"   : "normal",
+				(style & VTE_DRAW_ITALIC) ? "italic" : "regular");
 
-	g_return_val_if_fail (draw->font != NULL, FALSE);
+	g_return_val_if_fail (draw->fonts[VTE_DRAW_NORMAL] != NULL, FALSE);
 
-	uinfo = font_info_get_unistr_info (bold ? draw->font_bold : draw->font, c);
+	uinfo = font_info_get_unistr_info (draw->fonts[style], c);
 	return !uinfo->has_unknown_chars;
 }
 
 gboolean
 _vte_draw_char (struct _vte_draw *draw,
 	       struct _vte_draw_text_request *request,
-	       const PangoColor *color, guchar alpha, gboolean bold)
+	       const PangoColor *color, guchar alpha, guint style)
 {
 	gboolean has_char;
 
 	_vte_debug_print (VTE_DEBUG_DRAW,
-			"draw_char ('%c', color=(%d,%d,%d,%d), %s)\n",
+			"draw_char ('%c', color=(%d,%d,%d,%d), %s, %s)\n",
 			request->c,
 			color->red, color->green, color->blue,
-			alpha, bold ? "bold" : "normal");
+			alpha,
+			(style & VTE_DRAW_BOLD)   ? "bold"   : "normal",
+			(style & VTE_DRAW_ITALIC) ? "italic" : "regular");
 
-	has_char =_vte_draw_has_char (draw, request->c, bold);
+	has_char =_vte_draw_has_char (draw, request->c, style);
 	if (has_char)
-		_vte_draw_text (draw, request, 1, color, alpha, bold);
+		_vte_draw_text (draw, request, 1, color, alpha, style);
 
 	return has_char;
 }
@@ -1141,7 +1203,7 @@ _vte_draw_draw_rectangle (struct _vte_draw *draw,
 
 	cairo_set_operator (draw->cr, CAIRO_OPERATOR_OVER);
 	cairo_rectangle (draw->cr, x+VTE_LINE_WIDTH/2., y+VTE_LINE_WIDTH/2., width-VTE_LINE_WIDTH, height-VTE_LINE_WIDTH);
-	set_source_color_alpha (draw->cr, color, alpha);
+	_vte_draw_set_source_color_alpha (draw, color, alpha);
 	cairo_set_line_width (draw->cr, VTE_LINE_WIDTH);
 	cairo_stroke (draw->cr);
 }
@@ -1161,6 +1223,6 @@ _vte_draw_fill_rectangle (struct _vte_draw *draw,
 
 	cairo_set_operator (draw->cr, CAIRO_OPERATOR_OVER);
 	cairo_rectangle (draw->cr, x, y, width, height);
-	set_source_color_alpha (draw->cr, color, alpha);
+	_vte_draw_set_source_color_alpha (draw, color, alpha);
 	cairo_fill (draw->cr);
 }
