@@ -557,7 +557,7 @@ vte_sequence_handler_multiple(VteTerminal *terminal,
                               GValueArray *params,
                               VteTerminalSequenceHandler handler)
 {
-        vte_sequence_handler_multiple_limited(terminal, params, handler, G_MAXLONG);
+        vte_sequence_handler_multiple_limited(terminal, params, handler, G_MAXUSHORT);
 }
 
 static void
@@ -685,6 +685,11 @@ vte_sequence_handler_decset_internal(VteTerminal *terminal,
 		 GINT_TO_POINTER(0),
 		 GINT_TO_POINTER(MOUSE_TRACKING_ALL_MOTION_TRACKING),
 		 NULL, NULL,},
+		/* 1006: Extended mouse coordinates. */
+		{1006, &terminal->pvt->mouse_xterm_extension, NULL, NULL,
+		 GINT_TO_POINTER(FALSE),
+		 GINT_TO_POINTER(TRUE),
+		 NULL, NULL,},
 		/* 1010/rxvt: disallowed, scroll-on-output is set by user. */
 		{1010, NULL, NULL, NULL, NULL, NULL, NULL, NULL,},
 		/* 1011/rxvt: disallowed, scroll-on-keypress is set by user. */
@@ -1397,7 +1402,7 @@ vte_sequence_handler_dc (VteTerminal *terminal, GValueArray *params)
 static void
 vte_sequence_handler_DC (VteTerminal *terminal, GValueArray *params)
 {
-	vte_sequence_handler_multiple(terminal, params, vte_sequence_handler_dc);
+	vte_sequence_handler_multiple_r(terminal, params, vte_sequence_handler_dc);
 }
 
 /* Delete a line at the current cursor position. */
@@ -1523,7 +1528,7 @@ vte_sequence_handler_ec (VteTerminal *terminal, GValueArray *params)
 					*cell = screen->color_defaults;
 				} else {
 					/* Add new cells until we have one here. */
-					_vte_row_data_fill (rowdata, &screen->color_defaults, col);
+					_vte_row_data_fill (rowdata, &screen->color_defaults, col + 1);
 				}
 			}
 		}
@@ -1790,7 +1795,7 @@ vte_sequence_handler_reverse_index (VteTerminal *terminal, GValueArray *params)
 static void
 vte_sequence_handler_RI (VteTerminal *terminal, GValueArray *params)
 {
-	vte_sequence_handler_multiple(terminal, params, vte_sequence_handler_nd);
+	vte_sequence_handler_multiple_r(terminal, params, vte_sequence_handler_nd);
 }
 
 /* Save cursor (position). */
@@ -1856,9 +1861,7 @@ vte_sequence_handler_change_color (VteTerminal *terminal, GValueArray *params)
 				continue;
 
 			if (vte_parse_color (pairs[i + 1], &color)) {
-				terminal->pvt->palette[idx].red = color.red;
-				terminal->pvt->palette[idx].green = color.green;
-				terminal->pvt->palette[idx].blue = color.blue;
+                                _vte_terminal_set_color_internal(terminal, idx, &color);
 			} else if (strcmp (pairs[i + 1], "?") == 0) {
 				gchar buf[128];
 				g_snprintf (buf, sizeof (buf),
@@ -2296,6 +2299,9 @@ vte_sequence_handler_character_attributes (VteTerminal *terminal, GValueArray *p
 			terminal->pvt->screen->defaults.attr.half = 1;
 			terminal->pvt->screen->defaults.attr.bold = 0;
 			break;
+		case 3:
+			terminal->pvt->screen->defaults.attr.italic = 1;
+			break;
 		case 4:
 			terminal->pvt->screen->defaults.attr.underline = 1;
 			break;
@@ -2316,6 +2322,9 @@ vte_sequence_handler_character_attributes (VteTerminal *terminal, GValueArray *p
 			terminal->pvt->screen->defaults.attr.bold = 0;
 			terminal->pvt->screen->defaults.attr.half = 0;
 			break;
+		case 23:
+			terminal->pvt->screen->defaults.attr.italic = 0;
+			break;
 		case 24:
 			terminal->pvt->screen->defaults.attr.underline = 0;
 			break;
@@ -2534,6 +2543,74 @@ vte_sequence_handler_set_icon_and_window_title (VteTerminal *terminal, GValueArr
 	vte_sequence_handler_set_title_internal(terminal, params, TRUE, TRUE);
 }
 
+static void
+vte_sequence_handler_set_current_directory_uri (VteTerminal *terminal, GValueArray *params)
+{
+        GValue *value;
+        char *uri, *filename;
+
+        uri = NULL;
+        if (params != NULL && params->n_values > 0) {
+                value = g_value_array_get_nth(params, 0);
+
+                if (G_VALUE_HOLDS_POINTER(value)) {
+                        uri = vte_ucs4_to_utf8 (terminal, g_value_get_pointer (value));
+                } else if (G_VALUE_HOLDS_STRING(value)) {
+                        /* Copy the string into the buffer. */
+                        uri = g_value_dup_string(value);
+                }
+        }
+
+        /* Validate URI */
+        if (uri && uri[0]) {
+                filename = g_filename_from_uri (uri, NULL, NULL);
+                if (filename == NULL) {
+                        /* invalid URI */
+                        g_free (uri);
+                        uri = NULL;
+                } else {
+                        g_free (filename);
+                }
+        }
+
+        g_free(terminal->pvt->current_directory_uri_changed);
+        terminal->pvt->current_directory_uri_changed = uri;
+}
+
+static void
+vte_sequence_handler_set_current_file_uri (VteTerminal *terminal, GValueArray *params)
+{
+        GValue *value;
+        char *uri, *filename;
+
+        uri = NULL;
+        if (params != NULL && params->n_values > 0) {
+                value = g_value_array_get_nth(params, 0);
+
+                if (G_VALUE_HOLDS_POINTER(value)) {
+                        uri = vte_ucs4_to_utf8 (terminal, g_value_get_pointer (value));
+                } else if (G_VALUE_HOLDS_STRING(value)) {
+                        /* Copy the string into the buffer. */
+                        uri = g_value_dup_string(value);
+                }
+        }
+
+        /* Validate URI */
+        if (uri && uri[0]) {
+                filename = g_filename_from_uri (uri, NULL, NULL);
+                if (filename == NULL) {
+                        /* invalid URI */
+                        g_free (uri);
+                        uri = NULL;
+                } else {
+                        g_free (filename);
+                }
+        }
+
+        g_free(terminal->pvt->current_file_uri_changed);
+        terminal->pvt->current_file_uri_changed = uri;
+}
+
 /* Restrict the scrolling region. */
 static void
 vte_sequence_handler_set_scrolling_region (VteTerminal *terminal, GValueArray *params)
@@ -2782,8 +2859,7 @@ vte_sequence_handler_insert_lines (VteTerminal *terminal, GValueArray *params)
 {
 	GValue *value;
 	VteScreen *screen;
-	long param, end, row;
-	int i;
+	long param, end, row, i, limit;
 	screen = terminal->pvt->screen;
 	/* The default is one. */
 	param = 1;
@@ -2801,7 +2877,13 @@ vte_sequence_handler_insert_lines (VteTerminal *terminal, GValueArray *params)
 	} else {
 		end = screen->insert_delta + terminal->row_count - 1;
 	}
-	/* Insert the new lines at the cursor. */
+
+	/* Only allow to insert as many lines as there are between this row
+         * and the end of the scrolling region. See bug #676090.
+         */
+        limit = end - row + 1;
+        param = MIN (param, limit);
+
 	for (i = 0; i < param; i++) {
 		/* Clear a line off the end of the region and add one to the
 		 * top of the region. */
@@ -2822,8 +2904,7 @@ vte_sequence_handler_delete_lines (VteTerminal *terminal, GValueArray *params)
 {
 	GValue *value;
 	VteScreen *screen;
-	long param, end, row;
-	int i;
+	long param, end, row, i, limit;
 
 	screen = terminal->pvt->screen;
 	/* The default is one. */
@@ -2842,6 +2923,13 @@ vte_sequence_handler_delete_lines (VteTerminal *terminal, GValueArray *params)
 	} else {
 		end = screen->insert_delta + terminal->row_count - 1;
 	}
+
+        /* Only allow to delete as many lines as there are between this row
+         * and the end of the scrolling region. See bug #676090.
+         */
+        limit = end - row + 1;
+        param = MIN (param, limit);
+
 	/* Clear them from below the current cursor. */
 	for (i = 0; i < param; i++) {
 		/* Insert a line at the end of the region and remove one from
